(* file: tpdb.yacc *)
(* description: parser generator *)
(* author: Takahito Aoto *)

local 
    structure U = Util
    structure L = List
in
datatype tpdb_decl = STATES of string list 
		   | ALPHABET of string list
		   | TRANSITIONS of ((string * string) * string list) list
		   | START of string
		   | ACCEPT of string list

type tpdb_result = string list * string list * ((string * string) * string list) list * string list * string list

exception Error

(* 줾ΥѡȤʣȤϡ union Ȥ *)
fun getCat f xss = let val yss = L.mapPartial f xss in U.mapAppend (fn x=>x) yss end
val getStates = getCat (fn top => case top of STATES xs => SOME xs | _ => NONE) 
val getAlphabet = getCat (fn top => case top of ALPHABET xs => SOME xs | _ => NONE) 
val getTransitions = getCat (fn top => case top of TRANSITIONS xs => SOME xs | _ => NONE) 
val getStart = getCat (fn top => case top of START x => SOME [x] | _ => NONE) 
val getAccept = getCat (fn top => case top of ACCEPT xs => SOME xs | _ => NONE) 

fun getDecl decls = 
  let (* ٤ƤΥѡȤ뤫Υå *)
      val _ = if L.exists  (fn top => case top of STATES xs => true | _  => false) decls
              then () else (print "STATES section is not declared\n"; raise Error)
      val _ = if L.exists  (fn top => case top of ALPHABET xs => true | _  => false) decls
              then () else (print "ALPHABET section is not declared\n"; raise Error)
      val _ = if L.exists  (fn top => case top of TRANSITIONS xs => true | _  => false) decls
              then () else (print "TRANSITIONS section is not declared\n"; raise Error)
      val _ = if L.exists  (fn top => case top of START xs => true | _  => false) decls
              then () else (print "START section is not declared\n"; raise Error)
      val _ = if L.exists  (fn top => case top of ACCEPT xs => true | _  => false) decls
              then () else (print "ACCEPT section is not declared\n"; raise Error)
  in (getStates decls, getAlphabet decls, getTransitions decls, getStart decls, getAccept decls)
  end

end 
(* 桼 *)
%%
%name		Tpdb
%term           ID of string
              | STR of string
              | LPAREN
              | RPAREN
              | COMMA
              | ARROW
              | STATES
              | ALPHABET
              | TRANSITIONS
              | ACCEPT
              | START
              | EOF 

%nonterm	XstartX of tpdb_result
              | XspecX of tpdb_decl list
              | XidlistX of string list
              | XrulelistX of ((string * string) * string list) list
              | XruleX of (string * string) * string list
              | XanylistX of unit
              | XanyX of unit
%pos	   	int
%eop		EOF
%noshift	EOF

%%
XstartX:      XspecX                       (getDecl XspecX)
XspecX:       LPAREN STATES XidlistX RPAREN XspecX           (STATES XidlistX::XspecX)
            | LPAREN ALPHABET XidlistX RPAREN XspecX         (ALPHABET XidlistX::XspecX)
            | LPAREN TRANSITIONS XrulelistX RPAREN XspecX    (TRANSITIONS XrulelistX::XspecX)
            | LPAREN START ID RPAREN XspecX                (START ID::XspecX)
            | LPAREN ACCEPT XidlistX RPAREN XspecX            (ACCEPT XidlistX::XspecX)
            | LPAREN ID XanylistX RPAREN XspecX              (XspecX)
            |                              ([])
XidlistX:     ID XidlistX                  (ID::XidlistX)
            |                              ([])
XrulelistX:    XruleX  XrulelistX          (XruleX::XrulelistX)
            |                              ([])
XruleX:       LPAREN ID COMMA ID RPAREN ARROW XidlistX     ((ID1,ID2),XidlistX)
XanylistX:    XanyX XanylistX              ()
            |                              ()
XanyX:        ID                           ()
            | LPAREN XanylistX RPAREN      ()
            | ARROW                        ()
            | COMMA                        ()
            | STATES                       ()
            | ALPHABET                     ()
            | TRANSITIONS                   ()
            | START                      ()
            | ACCEPT                        ()
